#------------------------------------------------------------------
# $Copyright: Copyright (c) 2011 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Header: /project/isv-agent/VProAgents/OracleAppsCM/OracleAppsCM.pm,v 1.21 2010/08/11 09:37:10 ssrivast Exp $
#
# $Author: ssrivast $
# $Date: 2010/08/11 09:37:10 $
# $Revision: 1.21 $
#
# Provides:                            OracleAppsCM
# Author:                              Ashish Prabhu
# Creation Date:                       23 Feb, 2006
# Modification History:
# Known bugs/caveats:
#------------------------------------------------------------------


#------------
# Be strict..
#------------
use strict;

#----------------------
# Declare the package..
#----------------------
package OracleAppsCM;

use IO::File;
use IO::Handle;
use Socket;

use POSIX qw ( tmpnam );

my ( $CLUS_HOME, $sPrefix ) = ();

@::gsLogString = ();
%::hOracleAppsCMTypeValues  =  ();

use constant EPOP              =>  3;
use constant NO_DELAY          =>  0;
use constant DELAY             =>  1;

#---------------------
# Setup VCS/VAD home..
#---------------------
if ( $ENV{CLUSTER_HOME}  )  {
  $CLUS_HOME ||= $ENV { VAD_HOME } ||= "/opt/VRTSvad";
  $sPrefix = 'vad';
}
else { # VCS
  $CLUS_HOME ||= $ENV { VCS_HOME } ||= "/opt/VRTSvcs";
  $sPrefix = 'vcs';
}

#-----------------------------------------
# Define module-vendor specific commands..
#-----------------------------------------
my $sCmdHares  = "$CLUS_HOME/bin/hares";
my $sCmdHatype = "$CLUS_HOME/bin/hatype";
my $sCmdVcsDecrypt = "$CLUS_HOME/sbin/${sPrefix}decrypt";

#-------------------------------------
# Include required framework modules..
#-------------------------------------
use Sys;            # Support generalized UNIX commands
use Proc;           # For calls to the process routines
use Comms;          # For calls to socket(3C) routines
use Version;        # Version display generator
use VCSagentFW;     # Primary Agent Interface to the ACC Library
use VCSDefines;     # This contains constants that VCS uses

#--------------------------------
# Define OracleAppsCM subroutines..
#--------------------------------

=pod

#-------------------------------------------------------------------------
 Function :  bool ArgsValid ( IN String EntryPointName,
                              IN Integer SecondLevelMonitor,
                              IN String User,
                              IN String OracleHome,
                              IN String ScriptHome,
                              IN String SQLDIR,
                              IN String EnvFile,
                              IN String DB_USER,
                              IN String DB_PASSWD,
                              IN String MonitorProgram,
                              IN String VirtualHostname,
                              IN String ProcMon );

 Description : Validates the ArgList passed to the entry point.

 Parameters :

 $sEntryPointName               Entry pt name
 $localSecondLevelMonitor       Cycle count to run in-depth monitoring
 $localUser                     The User to start/stop the Concurrent Manager
 $localOracleHome               Pathname of the ORACLE_HOME directory
 $localScriptHome               Pathname of the Scripts directory
 $localSQLDIR                   Pathname of the sql scripts directory
 $localEnvFile                  Full pathname to the env file to setup the env
 $localDB_USER                  Username to connect to the database
 $localDB_PASSWD                Encrypted password of the database user
 $localMonitorProgram           External Monitor Program
 $localVirtualHostname          Virtual hostname associated with this CM instance
 $localProcMon                  List of process to be monitored
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
#-------------------------------------------------------------------------

=cut

sub ArgsValid {
  #----------------------------------------
  # Set up local vars from passed in ones..
  #----------------------------------------
  my ( $sEntryPointName,
       $localSecondLevelMonitor,
       $localUser,
       $localOracleHome,
       $localScriptHome,
       $localSQLDIR,
       $localEnvFile,
       $localDB_USER,
       $localDB_PASSWD,
       $localMonitorProgram,
       $localVirtualHostname,
       $localProcMon ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20001, "Subroutine <%s> called with:
  EntryPointName           [%s]
  SecondLevelMonitor       [%s]
  User                     [%s]
  OracleHome               [%s]
  ScriptHome               [%s]
  SQLDIR                   [%s]
  EnvFile                  [%s]
  DB_USER                  [%s]
  DB_PASSWD                [%s]
  MonitorProgram           [%s]
  VirtualHostname          [%s]",

  $sMyName, $sEntryPointName, $localSecondLevelMonitor, $localUser,
  $localOracleHome,  $localScriptHome, $localSQLDIR,  $localEnvFile,  
  $localDB_USER,  $localDB_PASSWD, $localMonitorProgram, $localVirtualHostname );

  #-------------------------------------
  # Set return code to hopeful success..
  #-------------------------------------
  my $iReturnCode = 1;

  #--------------------------------------------
  # Create hash to weed out empty args values..
  #--------------------------------------------
  my ( @lsMissing, %hsArgList ) = ();
  my @lsArgs =
  qw ( EntryPointName  SecondLevelMonitor  User  OracleHome
       ScriptHome SQLDIR EnvFile DB_USER DB_PASSWD MonitorProgram VirtualHostname );

  @hsArgList { @lsArgs } = @_; # Assign..

  #----------------------------------------
  # Reqd. attributes based on entry point..
  #----------------------------------------
  my @lsOnlineArgs  = qw ( User OracleHome DB_USER DB_PASSWD EnvFile 
                           VirtualHostname ScriptHome SQLDIR );
  my @lsOfflineArgs = qw ( User DB_USER DB_PASSWD ScriptHome EnvFile );
  my @lsMonitorArgs = qw ( User DB_USER DB_PASSWD EnvFile SQLDIR OracleHome );
  my @lsCleanArgs   = qw ( User DB_USER DB_PASSWD ScriptHome EnvFile );

  my %hlsNeeded = ( 'online'  => \@lsOnlineArgs,
                    'offline' => \@lsOfflineArgs,
                    'monitor' => \@lsMonitorArgs,
                    'clean'   => \@lsCleanArgs );

  my @lsReqdArgs = @{$hlsNeeded { $sEntryPointName }};

  #-----------------------
  # Check what's missing..
  #-----------------------
  if ( @lsMissing = grep { $hsArgList { $_ } !~ /\S/o } @lsReqdArgs ) {
    &logError ( 20002,
    "<%s> The following mandatory fields were not set [%s], Total [%s]",
    $sMyName, join ( ' ', @lsMissing ), scalar @lsMissing );
    $iReturnCode = 0; # Failure..
  }

  if (! $localProcMon )
  {
    &logError ( 20003, "<%s> The ProcMon attribute must not be NULL.", $sMyName);
    $iReturnCode = 0; # Failure..
  }

  $iReturnCode || return $iReturnCode; # Return, if failed..

  #-----------------------------------------------------------------------------
  # Agent specific validation starts
  #-----------------------------------------------------------------------------

  if ( $sEntryPointName eq "online"  ||
       $sEntryPointName eq "offline" )
  {
    if (! -d $localOracleHome )
    {
      &logError(20004,"<%s>:OracleHome [%s] is not accessible.",
        $sMyName,$localOracleHome);
      $iReturnCode = 0 ;
    }

    my $spwname = getpwnam($localUser);
    if ( !defined($spwname) )
    {
      &logError(20005, "<%s>:User name [%s] invalid.",$sMyName,$localUser);
      $iReturnCode = 0 ;
    }
    if (! -e $localScriptHome )
    {
      &logError(20006,"<%s>:ScriptHome [%s] is not accessible.",
        $sMyName,$localScriptHome);
      $iReturnCode = 0 ;
    }
    if (! -e $localSQLDIR )
    {
      &logError(20007,"<%s>:SQLDIR [%s] is not accessible.",
        $sMyName,$localSQLDIR);
      $iReturnCode = 0 ;
    }
  }
    #-----------------------------------------------------------------------------
    # Validate MonitorProgram if passed during online
    #-----------------------------------------------------------------------------
    if ( $sEntryPointName eq "online" )
    {
      if ( $localMonitorProgram =~ /(\S+)/ && 
               ! -x $1 )
      {
          &logError(20008,"<%s> MonitorProgram [%s] not executable.
                                Check the full pathname and the permissions",
                                $sMyName, $localMonitorProgram);
          $iReturnCode = 0 ;
      }
    }

  $iReturnCode ||
  &logWarn ( 20009,
  "<%s> Some of the attributes could not be validated. Returning [%s]",
  $sMyName, $iReturnCode );

  return $iReturnCode ;
}

=pod

 -------------------------------------------------------------------------
 Function : bool Start ( IN String State,
                         IN String IState,
                         IN String ResourceName,
                         IN String ResourceType,
                         IN String User,
                         IN String OracleHome,
                         IN String DB_USER,
                         IN String DB_PASSWD,
                         IN String EnvFile,
                         IN String VirtualHostname,
                         IN String ScriptHome,
                         IN String MonitorProgram,
                         IN String SQLDIR,
                         IN Integer ExpirationTime,
                         IN String ProcMon );

 Description : Start logic needed for online entry pt

 Parameters :

 $iState                    VCS State of the resource
 $iIState                   VCS Intermidiate State of the resource
 $sResourceName             Name of Resource to Start
 $sResourceType             Resource Type to Start
 $sUser                     The User to start/stop the Concurrent Manager
 $sOracleHome               Pathname of the ORACLE_HOME directory
 $sDB_USER                  Username to connect to the database
 $DB_PASSWD                
 $sEnvFile                  Full pathname to the env file to setup the env
 $sVirtualHostname          Virtual hostname associated with this CM instance
 $sScriptHome               Pathname of the Scripts directory
 $sMonitorProgram           Full path to external monitor program
 $sSQLDIR                   Pathname of the sql scripts directory
 $iExpirationTime
 $rlsProcMon
 
 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Start {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iState,
       $iIState,
       $sResourceName,
       $sResourceType,
       $sUser,
       $sOracleHome,
       $sDB_USER,
       $sDB_PASSWD,
       $sEnvFile,
       $sVirtualHostname,
       $sScriptHome,
       $sMonitorProgram,
       $sSQLDIR,
       $iExpirationTime,
       $rlsProcMon) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace( 20010, "Subroutine <%s> called with:
  User                     [%s]
  OracleHome               [%s]
  DB_USER                  [%s]
  DB_PASSWD                [%s]
  EnvFile                  [%s]
  VirtualHostname          [%s]
  ScriptHome               [%s]
  MonitorProgram           [%s]
  SQLDIR                   [%s]
  ExpirationTime           [%s]",
  $sMyName, $sUser, $sOracleHome, $sDB_USER, $sDB_PASSWD, $sEnvFile,    
  $sVirtualHostname, $sScriptHome, $sMonitorProgram, $sSQLDIR, $iExpirationTime );
  
  my $sDecryptedPassword = DecryptPassword($sDB_PASSWD);

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  my $sStartScript  = "$sScriptHome/adcmctl.sh" ;  
  my $sStartOptions = "start $sDB_USER/$sDecryptedPassword"; 
  my $iCommandExitCode = 0;
  my @lsCommandOut = ();
  my $bTimedOut;
  my $iTimeOut = ($iExpirationTime -time -10) ;

  $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut( $sEnvFile,
                                                          0,
                                                          $sStartScript,
                                                          $sStartOptions,
                                                          \$iCommandExitCode,
                                                          $sUser,
                                                          \@lsCommandOut,
                                                          \$bTimedOut,
                                                          $iTimeOut,
                                                          "",
                                                          "");

  if ( $iReturnCode )
  {
    #---------------------------------------------------------------------------
    # Command got executed.
    #---------------------------------------------------------------------------
    if ( $iCommandExitCode )
    {
      #-------------------------------------------------------------------------
      # Command failed.
      #-------------------------------------------------------------------------
      &logError (20011,"<%s>:[%s] with args [%s] returned exit code [%s]",
                       $sMyName,$sStartScript,$sStartOptions,$iCommandExitCode);
      return 0;
    }
    else
    {
      #-------------------------------------------------------------------------
      # Command executed successfully.
      #-------------------------------------------------------------------------
      &logTrace (20012, "<%s>:[%s] executed successfully.",
                         $sMyName, $sStartScript);
    }
  }
  else
  {
    if ( $bTimedOut )
    {
      #-------------------------------------------------------------------------
      # Command timed out.
      #-------------------------------------------------------------------------
      &logError (20013,"<%s>:[%s] did not complete within [%s] seconds.",
                 $sMyName,$sStartScript,$iTimeOut);
      return 0;
    }
    else
    {
      #-------------------------------------------------------------------------
      # Command failed.
      #-------------------------------------------------------------------------
      &logError (20014,"<%s>:[%s] failed.", $sMyName,$sStartScript);
      return 0;
    }
  }

  #-------------------------------------------------------------------------
  # Now delay the entry point if we have time to spare, so we
  # can guarantee that when first monitor is called, it will pass
  #-------------------------------------------------------------------------

  while ( ( $iTimeOut = ( $iExpirationTime - time ) ) > 10  )
  {
       my @liOnlinePids = ();
       my $iReturnCode = OracleAppsCM::GetProcState ( $iState,
                                                    $iIState,
                                                    $sUser,
                                                    $iExpirationTime ,
                                                    NO_DELAY,
                                                    $rlsProcMon,
                                                    \@liOnlinePids);

       if( VCSDef_MonitorExit::ONLINE == $iReturnCode ) {
         &logTrace ( 20015, "<%s> All processes are up ", $sMyName );
         return 1;
       }
       else {
         &logTrace ( 20016, "<%s> All required processes not up yet.
                            Time remaining [%s]", $sMyName, $iTimeOut);
        }
      sleep (5);
  }
  &logWarn ( 20017, "<%s> All processes not up after delay entry point ",
                             $sMyName );

  return 0;
}

=pod

 -------------------------------------------------------------------------
 Function : bool Stop ( IN Integer State,
                        IN Integer IState,
                        IN String User,
                        IN String OracleHome,
                        IN String DB_USER,
                        IN String DB_PASSWD,
                        IN String ScriptHome,
                        IN String EnvFile ,
                        IN Integer ExpirationTime,
                        IN String ProcMon);

 Description : Stop logic needed for offline entry pt

 Parameters :

 $iState                    VCS State of the resource
 $iIState                   VCS Intermidiate State of the resource
 $sUser                     The User to start/stop the Concurrent Manager
 $sOracleHome               Oracle Home Directory Path
 $sDB_USER                  Username to connect to the database
 $DB_PASSWD                
 $sScriptHome               Pathname of the Scripts directory
 $sEnvFile                  Full pathname to the env file to setup the env
 $iExpirationTime           Expiration Time
 $rlsProcMon                List of Processes to be monitored  
 

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Stop {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iState,
       $iIState,
       $sUser,
       $sOracleHome,
       $sDB_USER,
       $sDB_PASSWD,
       $sScriptHome,
       $sEnvFile,
       $iExpirationTime,
       $rlsProcMon ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20018, "Subroutine <%s> called with:
  User                     [%s]
  DB_USER                  [%s]
  DB_PASSWD                [%s]
  ScriptHome               [%s]
  EnvFile                  [%s]",

  $sMyName,   $sUser,   $sDB_USER,    $sDB_PASSWD,
  $sScriptHome, $sEnvFile );

  my $sDecryptedPassword = DecryptPassword($sDB_PASSWD);
  &logTrace( 20019, "Subroutine <%s> called with: passwd [%s] [%s]",
                    $sMyName, $sDB_PASSWD, $sDecryptedPassword);

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  my $sStopScript  = "$sScriptHome/adcmctl.sh" ;  
  my $sStopOptions = "stop $sDB_USER/$sDecryptedPassword"; 
  my $iCommandExitCode = 0;
  my @lsCommandOut = ();
  my $bTimedOut;
  my $iTimeOut = ($iExpirationTime -time -10) ;

  $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut( $sEnvFile,
                                                          0,
                                                          $sStopScript,
                                                          $sStopOptions,
                                                          \$iCommandExitCode,
                                                          $sUser,
                                                          \@lsCommandOut,
                                                          \$bTimedOut,
                                                          $iTimeOut,
                                                          "",
                                                          "");

  if ( $iReturnCode )
  {
    #---------------------------------------------------------------------------
    # Command got executed.
    #---------------------------------------------------------------------------
    if ( $iCommandExitCode )
    {
      #-------------------------------------------------------------------------
      # Command failed.
      #-------------------------------------------------------------------------
      &logError (20020,"<%s>:[%s] with args [%s] returned exit code [%s]",
                       $sMyName,$sStopScript,$sStopOptions,$iCommandExitCode);
      return 0;
    }
    else
    {
      #-------------------------------------------------------------------------
      # Command executed successfully.
      #-------------------------------------------------------------------------
      &logTrace (20021, "<%s>:[%s] executed successfully.",
                         $sMyName, $sStopScript);
    }
  }
  else
  {
    if ( $bTimedOut )
    {
      #-------------------------------------------------------------------------
      # Command timed out.
      #-------------------------------------------------------------------------
      &logError (20022,"<%s>:[%s] did not complete within [%s] seconds.",
                 $sMyName,$sStopScript,$iTimeOut);
      return 0;
    }
    else
    {
      #-------------------------------------------------------------------------
      # Command failed.
      #-------------------------------------------------------------------------
      &logError (20023,"<%s>:[%s] failed.", $sMyName,$sStopScript);
      return 0;
    }
  }


  while ( ( $iTimeOut = ( $iExpirationTime - time ) ) > 10  )
  {
       my @liOnlinePids = ();
       my $iReturnCode = OracleAppsCM::GetProcState ( $iState,
                                                    $iIState,
                                                    $sUser,
                                                    $iExpirationTime ,
                                                    DELAY,
                                                    $rlsProcMon,
                                                    \@liOnlinePids);

       if( VCSDef_MonitorExit::OFFLINE == $iReturnCode ) {
         &logTrace ( 20024, "<%s> All processes are down ", $sMyName );
         return 1;
       }
       else {
         &logTrace ( 20025, "<%s> All processes not down yet.
                            Time remaining [%s]", $sMyName, $iTimeOut);
        }
      sleep (5);
  }
  &logWarn ( 20026, "<%s> All processes not down after delay entry point ",
                             $sMyName );


  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : bool Kill ( IN Integer State,
                        IN Integer IState,
                        IN String User,
                        IN String DB_USER,
                        IN String DB_PASSWD,
                        IN String ScriptHome,
                        IN String EnvFile,
                        IN Integer CleanReason, 
                        IN Integer ExpirationTime,
                        IN String ProcMon );

 Description : Forces shutdown of resource needed for clean entry point

 Parameters :

 $iState                    VCS State of the resource
 $iIState                   VCS Intermidiate State of the resource
 $sUser                     The User to start/stop the Concurrent Manager
 $sDB_USER                  Username to connect to the database
 $DB_PASSWD
 $sScriptHome               Pathname of the Scripts directory
 $sEnvFile                  Full pathname to the env file to setup the env
 $iCleanReason              Clean Reason 
 $iExpirationTime           Expiration Time
 $rlsProcMon                List of Processes to be monitored

 Returns : boolean value in the set of {0,1} where
           0 false
           1 true
 -------------------------------------------------------------------------

=cut

sub Kill {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iState,
       $iIState,
       $sUser,
       $sDB_USER,
       $sDB_PASSWD,
       $sScriptHome,
       $sEnvFile,
       $iCleanReason,
       $iExpirationTime,
       $rlsProcMon) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20027, "Subroutine <%s> called with:
  User                     [%s]
  DB_USER                  [%s]
  ScriptHome               [%s]
  EnvFile                  [%s]
  CleanReason              [%s]",

  $sMyName, $sUser, $sDB_USER, $sScriptHome, $sEnvFile , $iCleanReason);

  #--------------------------------------
  # Set Return Code Optimistically to 1..
  #--------------------------------------
  my $iReturnCode = 1;

  #-----------------------------
  # Get all the pids and kill them
  #-----------------------------

  my @liOnlinePids = ();

  my $iStatus_ReturnCode = OracleAppsCM::GetProcState ($iState,
                                                $iIState,
                                                $sUser, 
                                                $iExpirationTime,
                                                NO_DELAY,
                                                $rlsProcMon,
                                                \@liOnlinePids);

  if ( VCSDef_MonitorExit::ONLINE == $iStatus_ReturnCode )
  {
    &logTrace(20028,"<%s>:Oracle Concurrent Manager processes running", $sMyName);

    my $blongproclist = 1;

  }
  if ( VCSDef_MonitorExit::OFFLINE == $iStatus_ReturnCode )
  {
    &logTrace(20029,"<%s>:Oracle Concurrent Manager processes not running", $sMyName);
  }

  #-----------------------------------
  # Kill processes if they are running
  #-----------------------------------
  foreach my $ipid ( @liOnlinePids )
  {
      #---------------------------------------------------------------------
      # Kill the parent and all the children of Concurrent Manager processes
      #---------------------------------------------------------------------
      Proc::KillPIDandAllChildren( $ipid )
  }

  return $iReturnCode;
}

=pod

 -------------------------------------------------------------------------
 Function : int GetResourceState ( IN Integer State, 
                                   IN Integer IState, 
                                   IN String User, 
                                   IN String VirtualHost,
                                   IN Integer ExpirationTime,
                                   IN Integer Delay,
                                   IN String ProcMon);

 Description : First level monitor to determine state of resource.

 Parameters :

 $iState                    VCS State of the resource
 $iIState                   VCS Intermidiate State of the resource
 $sUser                     The User to start/stop the Concurrent Manager
 $sVirtualHost              Virtual hostname associated with this CM instance
 $iExpirationTime           Amount of time left to execute this procedure
 $iDelay                    Delay
 $rlsProcMon                List of CM Processes to monitor
 

 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50   VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub GetResourceState {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iState, 
       $iIState, 
       $sUser, 
       $sVirtualHost,
       $iExpirationTime ,
       $iDelay,
       $rlsProcMon) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20030, "Subroutine <%s> called:
  User [%s] VirtualHost [%s]", $sMyName, $sUser, $sVirtualHost);

  #------------------------------------------------------
  # Do an IP check. This  is a quick check on the offline 
  # node
  #------------------------------------------------------

  if (  Comms::IsIPLocal ($sVirtualHost) ) {
    &logTrace ( 20031, "<%s> plumbed locally [%s]", 
                $sMyName, $sVirtualHost);
  }
  else {
    &logTrace ( 20032, "<%s> not plumbed locally [%s]", 
                $sMyName, $sVirtualHost);
     return ( VCSDef_MonitorExit::OFFLINE );
  }

  #-----------------------------
  # Do first level Process Check 
  #-----------------------------

  my @liOnlinePids = ();
  my $iReturnCode = OracleAppsCM::GetProcState ($iState,
                                                $iIState,
                                                $sUser, 
                                                $iExpirationTime,
                                                $iDelay,
                                                $rlsProcMon,
                                                \@liOnlinePids);

  if ( VCSDef_MonitorExit::ONLINE == $iReturnCode ) {
    &logTrace ( 20033,
    "<%s> found all required processes",
    $sMyName );
  }
  else {
    &logTrace ( 20034,
    "<%s> did not find all required processes, returning [%s]",
    $sMyName, $iReturnCode );
    $iReturnCode = VCSDef_MonitorExit::OFFLINE;
  }

  &logTrace ( 20035, "<%s> returning [%s]", $sMyName, $iReturnCode );

  return ( $iReturnCode );
}


=pod

 -------------------------------------------------------------------------
 Function : int GetProcState ( IN Integer State,
                               IN Integer IState,
                               IN String User,
                               IN Integer ExpirationTime,
                               IN Integer Delay,
                               IN String ProcMon,
                               IN String OnlinePids);

 Description : Check if all the CM processes are running. FNDSM will start 
               the missing processes, so we wait for sometime if FNDSM
               is running but the rest are missing. 
               
 Parameters :

 $iState                    VCS State of the resource
 $iIState                   VCS Intermidiate State of the resource
 $sUser                     The User to start/stop the Concurrent Manager
 $iExpirationTime           Amount of time left to execute this procedure
 $iDelay                    Delay
 $rlsProcMon                List of CM Processes to monitor
 $rliOnlinePids             Online process ids.

 Returns : Integer value as defined below:

 100  OFFLINE                                VCSDef_MonitorExit::OFFLINE
 105  ONLINE with confidence level 50        VCSDef_MonitorExit::PARTIAL_ONLINE
 110  ONLINE with confidence level 100       VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined  VCSDef_MonitorExit::UNKNOWN

#---------------------------------------------------------------
# FNDLIBR FND CPMGR exists ( except on Linux ) and FNDSM exists
#---------------------------------------------------------------
 

=cut

sub GetProcState {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iState,
       $iIState,
       $sUser, 
       $iExpirationTime,
       $iDelay,
       $rlsProcMon,
       $rliOnlinePids) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20036, "Subroutine <%s> called:
  User                     [%s]", $sMyName, $sUser  );

  my @hProcListHash = ();
  my $iReturnCode = VCSDef_MonitorExit::ONLINE;
  my $iNumberOfMatches = 0;

  my $iUid = getpwnam $sUser; 
  my $iProcessCount = Proc::GetLongProcessListHash(\@hProcListHash);

  if ( 0 == $iProcessCount )
  {
    #--------------------------------------------------------------------------
    # No processes found.
    #--------------------------------------------------------------------------
    &logError(20037,"<%s> : Error getting all running processes.",$sMyName);
    return ( $VCSDef_MonitorExit::OFFLINE);
  }

  my $sOS = $^O;

  my @hUidProcListHash = ();
  my $iMatchedPids = Proc::FindUidPids ( $iUid, 
                                         \@hProcListHash, 
                                         \@hUidProcListHash );

  &logTrace(20038,"<%s> : Number of processes for User %s = %s", 
                           $sMyName, $sUser, $iMatchedPids);

  #---------------------------------------------
  # Check whether the CM processes are running.
  #---------------------------------------------

  if ( 0 >= $iMatchedPids )
  {
   
    &logTrace(20039,"<%s> : Internal Concurrent Manager is down ", $sMyName);
    return VCSDef_MonitorExit::OFFLINE; # return immediately
  }


  my @hProcListHashMatched = ();
  my $bPartialOnline = 0;
  foreach my $sProc (@$rlsProcMon) 
  {
    my $sFilter = "^$sProc";
    &logTrace(20040,"<%s> : Process Filter set to [%s]", $sMyName, $sFilter);
    $iNumberOfMatches = 0 ;
    @hProcListHashMatched = ();
    $iNumberOfMatches = Proc::FilterProcs($sFilter,
                                          \@hUidProcListHash,
                                          \@hProcListHashMatched);

    if ( 0 == $iNumberOfMatches )
    {
      
        &logTrace(20041,"<%s> : Process [%s] is not running", $sMyName,$sProc);
        $iReturnCode = VCSDef_MonitorExit::OFFLINE;

    }
    else
    {
        &logTrace(20042,"<%s> : Process [%s] is Running",$sMyName,$sProc);
        $bPartialOnline = 1;

        #----------------------------------------------------------
        # Add the online pids to the outgoing pid list
        #----------------------------------------------------------

        my @liPidList=();
        Proc::GetValuesForKeyFromProcDataHash("PID"                 ,
                                              \@hProcListHashMatched,
                                              \@liPidList           );
        push(@$rliOnlinePids, @liPidList);

        #----------------------------------------------------------
        # Continue checking other processes
        #----------------------------------------------------------
    }
  }

  if ( (1 == $bPartialOnline) && (VCSDef_MonitorExit::OFFLINE == $iReturnCode) )
  {
    $iReturnCode = VCSDef_MonitorExit::PARTIAL_ONLINE;
  }

  return $iReturnCode;
}


=pod

 -------------------------------------------------------------------------
 Function : int Monitor ( IN Integer State,
                          IN Integer IState,
                          IN String ResourceName,
                          IN String ResourceType,
                          IN String User,
                          IN String OracleHome,
                          IN String DB_USER,
                          IN String DB_PASSWD,
                          IN String EnvFile,
                          IN String SQLDIR,
                          IN Integer SecondLevelMonitor,
                          IN String MonitorProgramAndArgs,
                          IN String VirtualHostname,
                          IN Integer ExpirationTime );


 Description: Performs the in-depth monitoring

 Parameters :

 $iState                    VCS State of the resource
 $iIState                   VCS Intermidiate State of the resource
 $sResourceName             Name of Resource to Start
 $sResourceType             Resource Type to Start
 $sUser                     The User to start/stop the Concurrent Manager
 $sOracleHome               Pathname of the ORACLE_HOME directory
 $sDB_USER                  Username to connect to the database
 $DB_PASSWD
 $sEnvFile                  Full pathname to the env file to setup the env
 $sSQLDIR                   Pathname of the sql scripts directory
 $iSecondLevelMonitor
 $sMonitorProgramAndArgs    Full path to external monitor program
 $sVirtualHostname          Virtual hostname associated with this CM instance
 $iExpirationTime

 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 100         VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------

=cut

sub Monitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iState,
       $iIState,
       $sResourceName,
       $sResourceType,
       $sUser,
       $sOracleHome,
       $sDB_USER,
       $sDB_PASSWD,
       $sEnvFile,
       $sSQLDIR,
       $iSecondLevelMonitor,
       $sMonitorProgramAndArgs, 
       $sVirtualHostname , 
       $iExpirationTime) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20043, "Subroutine <%s> called with:
  State                    [%s]
  IState                   [%s]
  ResourceName             [%s]
  ResourceType             [%s]
  User                     [%s]
  OracleHome               [%s]
  DB_USER                  [%s]
  DB_PASSWD                [%s]
  EnvFile                  [%s]
  SQLDIR                   [%s]
  ExpirationTime           [%s]",

  $sMyName, $iState, $iIState, $sResourceName, $sResourceType,
  $sUser, $sOracleHome, $sDB_USER, $sDB_PASSWD,$sEnvFile, $sSQLDIR,$iExpirationTime);

  #-----------------------------
  # Set return code to success..
  #-----------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # Success..

  if ( OracleAppsCM::RunSecondLevelMonitor ( $iSecondLevelMonitor,
                                           $sResourceType,
                                           $sResourceName ) )
  {
     $iReturnCode = OracleAppsCM::SecondLevelMonitor($iState,
                                                     $iIState,
                                                     $iExpirationTime,
                                                     $sUser,
                                                     $sSQLDIR,
                                                     $sDB_USER,
                                                     $sDB_PASSWD,
                                                     $sEnvFile,
                                                     $sOracleHome,
                                                     $sVirtualHostname);

     if ( VCSDef_MonitorExit::ONLINE == $iReturnCode )  {
       &logTrace ( 20044, "<%s> : Resource [%s] online after SLM ",
                           $sMyName, $sResourceName );
     }
     else {
       &logTrace ( 20045, "<%s> : Resource [%s] not online after SLM . Status = [%s]",
                           $sMyName, $sResourceName , $iReturnCode);
       return $iReturnCode;
     }

  }

  #----------------------------------------------------------
  # We are done monitoring if MonitorProgram is not defined
  #----------------------------------------------------------

  if ( $sMonitorProgramAndArgs eq "" ) {
    &logTrace ( 20046, "<%s> : MonitorProg is Null ", $sMyName);
    return $iReturnCode;
  }
 
  #--------------------------------------------------
  # Run external user provided monitor program
  #--------------------------------------------------
  my ($sMonitorProgram,$sMonitorProgramArgs) =
                           split(/\s/,$sMonitorProgramAndArgs,2);


  if ( ! -e $sMonitorProgram )
  {
      &logError(20047, "<%s>: MonitorProgram [%s] not executable.
                   Check existence and permissions.", $sMyName, $sMonitorProgram);

  }

  $sMonitorProgramArgs = "" if ( ! defined $sMonitorProgramArgs);


  $iReturnCode = OracleAppsCM::ExternalMonitor($iExpirationTime,
                                             $sUser,
                                             $sMonitorProgram,
                                             $sMonitorProgramArgs,
                                             $sOracleHome);

  if ( VCSDef_MonitorExit::ONLINE == $iReturnCode )  {
     &logTrace ( 20048, "<%s> : Resource [%s] online after external monitor ",
                         $sMyName, $sResourceName );
  }
  else {
    &logTrace ( 20049, "<%s> : Resource [%s] not online after external monitor. 
                       Status = [%s]", $sMyName, $sResourceName , $iReturnCode);
  }
  return $iReturnCode;
}


=pod
 -------------------------------------------------------------------------
 Function : int SecondLevelMonitor ( IN String State,
                                     IN String IState,
                                     IN String ExpirationTime,
                                     IN String User,
                                     IN String SQLDIR,
                                     IN String DB_USER,
                                     IN String DB_PASSWD,
                                     IN String EnvFile,
                                     IN String OracleHome,
                                     IN String VirtualHostname);

 Description : Runs the SecondLevelMonitor

 Parameters :

 $iState                    VCS State of the resource
 $iIState                   VCS Intermidiate State of the resource
 $iExpirationTime
 $sUser                     The User to start/stop the Concurrent Manager
 $sSQLDIR                   Pathname of the sql scripts directory
 $sDB_USER                  Username to connect to the database
 $DB_PASSWD
 $sEnvFile                  Full pathname to the env file to setup the env
 $sOracleHome               Pathname of the ORACLE_HOME directory
 $sVirtualHostname          Virtual hostname associated with this CM instance

 Returns  : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------
=cut

sub SecondLevelMonitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iState, 
       $iIState, 
       $iExpirationTime,
       $sUser,
       $sSQLDIR,
       $sDB_USER, 
       $sDB_PASSWD, 
       $sEnvFile,
       $sOracleHome,
       $sVirtualHostname ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace (20050,"<%s>: here 1", $sMyName );
  &logTrace ( 20051, "Subroutine <%s> called with:
  User                  [%s]
  SQLDIR                [%s]
  DB_USER               [%s]
  EnvFile               [%s]
  OracleHome            [%s]
  VirtualHostname       [%s]",

  $sMyName, $sUser, $sSQLDIR, $sDB_USER, $sEnvFile, $sOracleHome, $sVirtualHostname);
  &logTrace (20052,"<%s>: here 2", $sMyName );

  my $iExitCode = 0;
  my @lsCommandOut = ();
  my $bTimedOut;
  my $iTimeOut = ( $iExpirationTime - time - 10 ) ;

  my $sDecryptedPassword = DecryptPassword($sDB_PASSWD);

  my $sServerStatusCmd = "$sOracleHome/bin/sqlplus";
  my $sServerStatusArgs = "$sDB_USER/$sDecryptedPassword \@$sSQLDIR/afimchk.sql";

  &logTrace (20053,"<%s>: here 3", $sMyName );
  my $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut($sEnvFile,
                                                     "",
                                                     $sServerStatusCmd,
                                                     $sServerStatusArgs,
                                                     \$iExitCode,
                                                     $sUser,
                                                     \@lsCommandOut,
                                                     \$bTimedOut,
                                                     $iTimeOut,
                                                     "",
                                                     "");

  &logTrace (20054,"<%s>: here [%s]", $sMyName, $iReturnCode );

  if ( $iReturnCode )
  {
    #---------------------------------------------------------------------------
    # Command got executed.
    #---------------------------------------------------------------------------
    if ( $iExitCode )
    {
      #-------------------------------------------------------------------------
      # Command failed.
      #-------------------------------------------------------------------------
      &logError (20055,"<%s>: [%s] with args [%s] returned exit code [%s]",
                       $sMyName,$sServerStatusCmd,$sServerStatusArgs,$iExitCode);
      return 0;
    }
    else
    {
      #-------------------------------------------------------------------------
      # Command executed successfully.
      #-------------------------------------------------------------------------
      &logTrace (20056, "<%s>:[%s] executed successfully.",
                         $sMyName, $sServerStatusCmd);
    }
  }
  else
  {
    if ( $bTimedOut )
    {
      #-------------------------------------------------------------------------
      # Command timed out.
      #-------------------------------------------------------------------------
      &logError (20057,"<%s>:[%s] did not complete within [%s] seconds.",
                 $sMyName,$sServerStatusCmd,$iTimeOut);
    }
    else
    {
      #-------------------------------------------------------------------------
      # Command failed.
      #-------------------------------------------------------------------------
      &logError (20058,"<%s>:[%s] failed.", $sMyName,$sServerStatusCmd);
    }
  }
  my $sRunning = "Internal Conc Manager is running on";

  foreach my $sOutputLine ( @lsCommandOut )
  {
    &logTrace (20059,"<%s>: Found [%s] ", $sMyName,$sOutputLine);
    if ( $sOutputLine =~ /^$sRunning/ ) {
      &logTrace (20060,"<%s>: Found [%s] ", $sMyName,$sRunning);
      return VCSDef_MonitorExit::ONLINE;
    }
  }
}
=pod
 -------------------------------------------------------------------------
 Function : int ExternalMonitor ( IN Integer ExpirationTime,
                                  IN String User,
                                  IN String MonitorProgram,
                                  IN String MonitorArgs,
                                  IN String OracleHome );

 Description : Handles user defined external monitor program

 Parameters :

 $iExpirationTime           Expiration time
 $sUser                     The User to start/stop the Concurrent Manager
 $sMonitorProgram           Full path to external monitor program
 $sMonitorArgs              External Monitor program arguments.
 $sOracleHome               Pathname of the ORACLE_HOME directory

 Returns  : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 110          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------
=cut

sub ExternalMonitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iExpirationTime,
       $sUser,
       $sMonitorProgram,
       $sMonitorArgs,
       $sOracleHome ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20061, "Subroutine <%s> called:
  ExpirationTime    [%s]
  User              [%s]
  MonitorProgram    [%s]
  MonitorArgs       [%s] 
  OracleHome        [%s]",

  $sMyName, $iExpirationTime, $sUser, $sMonitorProgram, $sMonitorArgs, $sOracleHome );

  #---------------------------
  # Set Return Code to false..
  #---------------------------
  my $iReturnCode = VCSDef_MonitorExit::ONLINE; # set optimistically to ONLINE

  my $iMonitorProgramTimeout = $iExpirationTime - time - EPOP - EPOP ;

  if ( $iMonitorProgramTimeout < EPOP )
  {
     &logInfo (20062, "<%s> : Not enough time left to run the defined MonitorProgram [%s]",
                        $sMyName, $sMonitorProgram );
     return $iReturnCode;
  }

  &logTrace(20063,"<%s>:About to run [%s] with arguments [%s]",
                   $sMyName,$sMonitorProgram,$sMonitorArgs);


  my $iMonitorProgramExitCode = 0;
  my @lsCommandOut            = ();
  my $bTimedOut               = 0;


if (Sys::RunWithEnvCmdWithOutputWithTimeOut( "",
                                               0,
                                               $sMonitorProgram,
                                               $sMonitorArgs,
                                               \$iMonitorProgramExitCode,
                                               $sUser,
                                               \@lsCommandOut,
                                               \$bTimedOut,
                                               $iMonitorProgramTimeout,
                                               "",
                                               $sOracleHome))
  {
      #--------------------------------------------------------------------------
      # The MonitorProgram was successfully run; check the exit code it returned.
      #--------------------------------------------------------------------------
      if( ( VCSDef_MonitorExit::ONLINE == $iMonitorProgramExitCode ) ||
          ( 0 == $iMonitorProgramExitCode ) )
      {
        $iReturnCode = VCSDef_MonitorExit::ONLINE;
        &logTrace( 20064,"<%s>:MonitorProgram [%s] with args [%s] and user context [%s]
                          indicated ONLINE (exit code [%s])",
                            $sMyName,
                            $sMonitorProgram,
                            $sMonitorArgs,
                            $sUser,
                            $iMonitorProgramExitCode );

      }
      elsif( ( $iMonitorProgramExitCode == VCSDef_MonitorExit::OFFLINE ) ||
             ( 1 == $iMonitorProgramExitCode ) )
      {
        #----------------------------------------------------------------------
        # We must return PARTIAL_ONLINE to reflect that the first level process
        # check (and possibly the SecondLevelMonitor check) did succeed.
        #----------------------------------------------------------------------

        $iReturnCode = VCSDef_MonitorExit::PARTIAL_ONLINE;
        &logError( 20065,"<%s>:MonitorProgram [%s] with args [%s] and user context [%s]
                          indicated OFFLINE (exit code [%s])",
                            $sMyName,
                            $sMonitorProgram,
                            $sMonitorArgs,
                            $sUser,
                            $iMonitorProgramExitCode);
        VCSagentFW::logOutputFromCommand( "$sMonitorProgram $sMonitorArgs",
                                                  $iMonitorProgramExitCode,
                                                  \@lsCommandOut ,
                                                  LogInt::LLError);
        $iReturnCode = VCSDef_MonitorExit::PARTIAL_ONLINE;
      }

      else
      {
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
        &logWarn( 20066, "<%s>:MonitorProgram [%s] with args [%s] and user context [%s]
                           did not indicate ONLINE nor OFFLINE (it provided exit code [%s])",
                           $sMyName,
                           $sMonitorProgram,
                           $sMonitorArgs,
                           $sUser,
                           $iMonitorProgramExitCode);
        VCSagentFW::logOutputFromCommand( "$sMonitorProgram $sMonitorArgs",
                                                  $iMonitorProgramExitCode,
                                                  \@lsCommandOut,
                                                  LogInt::LLWarn);
      }
    }
    else
    {
      if( $bTimedOut )
      {
        #----------------------------------------------------------------------
        # Command timed out!  Return UNKNOWN as the first (and possibly second)
        # level monitoring did work -- it is considered safer to return UNKNOWN
        # as the service may be generally OK, and the MonitorProgram didn't
        # determine for sure that the service was not up, as the defined
        # methodology to do so (as implemented in the MonitorProgram) could not
        # be run successfully (it timed out).
        #----------------------------------------------------------------------
        &logError( 20067, "<%s>:MonitorProgram [%s] with args [%s] and user context [%s]
                            failed to run (timed out)",
                            $sMyName,
                            $sMonitorProgram,
                            $sMonitorArgs,
                            $sUser );
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
      }
      else
      {
        #----------------------------------------------------------------------
        # The MonitorProgram was not successfully run; The call to the
        # MonitorProgram failed (but did NOT timeout).  Return UNKNOWN for the
        # same reasons as explained above.
        #----------------------------------------------------------------------
        &logError( 20068, "<%s>:MonitorProgram [%s] with args [%s] and user context [%s]
                           failed to run )",
                            $sMyName,
                            $sMonitorProgram,
                            $sMonitorArgs,
                            $sUser );
        $iReturnCode = VCSDef_MonitorExit::UNKNOWN;
      }
  }
  return $iReturnCode;
}


=pod
----------------------------------------------------------------------------
 Function:  String DecryptPassword ( IN String CryptedPassword );

 Description : Decrypts the password using VCS decrypt subroutine.

 Parameters :
  $sCryptedPassword       Decrypts the password 

 Returns : None
----------------------------------------------------------------------------

=cut

sub DecryptPassword
{
  my ($sCryptedPassword ) = @_;

  my $sMyName = ( caller 0 )[3];
  &logTrace(20069,"Subroutine <%s> called.", $sMyName );

  my $sDecryptCmd = "/opt/VRTSvcs/sbin/vcsdecrypt";

  #-----------------------------------------------------------------
  # Check decrypt command and return immediately if not executable.
  #-----------------------------------------------------------------
  if ( ! -x $sDecryptCmd )
  {
    &logError( 20070, "<%s>:Decrypt command [%s] is not an executable.",
                       $sMyName, $sDecryptCmd);
    return 0;
  }

  #----------------------------------------
  # Execute the actual vcsdecrypt command.
  #----------------------------------------
  my $iExitCode;
  my @lsCommandOut;
  my $bTimedOut,
  my $iTimeOut = 5;
  my $iReturnCode;

  $iReturnCode = Sys::RunWithEnvCmdWithOutputWithTimeOut( "",
                                                          0,
                                                          $sDecryptCmd,
                                                          $sCryptedPassword,
                                                          \$iExitCode,
                                                          "root",
                                                          \@lsCommandOut,
                                                          \$bTimedOut,
                                                          $iTimeOut,
                                                          "",
                                                          "");


  if ( $iReturnCode )
  {
    #----------------------------------
    # Command executed.
    #----------------------------------
    if ( $iExitCode )
    {
      #-----------------
      # Command failed.
      #-----------------
      &logError (20071,"<%s>:[%s] failed with exit code [%s]",
                     $sMyName,$sDecryptCmd,$iExitCode);
      return ;
    }
    else
    {
      #---------------------------------
      # Command executed successfully.
      #---------------------------------
      &logTrace (20072, "<%s>:Command [%s] executed successfully.",
                 $sMyName, $sDecryptCmd);
      return $lsCommandOut[0] ;
    }
  }
  else
  {
    #-----------------------------------
    # Problem during command execution.
    #-----------------------------------
    if ( $bTimedOut )
    {
      #--------------------
      # Command timed out.
      #--------------------
      &logError (20073,"<%s>:[%s] did not complete within [%s] seconds.",
                 $sMyName,$sDecryptCmd,$iTimeOut);
      return;
    }
    else
    {
      #--------------------
      # Command failed.
      #--------------------
      &logError (20074,"<%s>:[%s] failed.", $sMyName,$sDecryptCmd);
    }
    return;
  }
}

=pod
 -------------------------------------------------------------------------
 Function : int RunSecondLevelMonitor ( IN Integer SecondLevelMonitor,
                                        IN String ResourceType,
                                        IN String ResourceName )

 Description: Checks whether time to run SecondLevelMonitor and initializes
              counter

 Parameters :
  $iSecondLevelMonitor
  $sResourceType
  $sResourceName

 Returns : Integer value as defined below:

 100  OFFLINE                                  VCSDef_MonitorExit::OFFLINE
 110  ONLINE with confidence level 100          VCSDef_MonitorExit::ONLINE
 99   UNKNOWN; State couldn't be determined    VCSDef_MonitorExit::UNKNOWN
 -------------------------------------------------------------------------
=cut

sub RunSecondLevelMonitor {
  #----------------
  # Get Arguments..
  #----------------
  my ( $iSecondLevelMonitor,
       $sResourceType,
       $sResourceName ) = @_;

  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20075, "Subroutine <%s> called:
  SecondLevelMonitor        [%s]
  ResourceType              [%s]
  ResourceName              [%s]",

  $sMyName, $iSecondLevelMonitor, $sResourceType, $sResourceName );

  #---------------------------
  # Set Return Code to false..
  #---------------------------
  my $iReturnCode = 0; # Do not run SLM..

  #------------------------
  # Setup local variables..
  #------------------------
  my $sSLMDir = "/tmp/.VRTS$sResourceType";
  my $sSLMFile = "$sSLMDir/$sResourceName.slm";
  my $iSLMCntr = undef;

  #--------------------------------------------
  # Validate SecondLevelMonitor trivial cases..
  #--------------------------------------------
  if ( 0 == $iSecondLevelMonitor ||
       1 == $iSecondLevelMonitor ) {
    $iReturnCode = $iSecondLevelMonitor; # Assign..

    &logTrace ( 20076,
    "<%s> Got request to remove SLM file [%s]. Returning with [%s]",
    $sMyName, $sSLMFile, $iSecondLevelMonitor );

    unlink $sSLMFile if ( -e $sSLMFile ); # Remove SLM file..

    return $iReturnCode;
  }

 #----------------------------------
  # Create history folder if needed..
  #----------------------------------
  if ( -d $sSLMDir ) {
    &logTrace ( 20077, "<%s> SLM folder [%s] exists",
    $sMyName, $sSLMDir );
  }
  else {
    mkdir ( $sSLMDir, 0700 ) ||
    &logWarn ( 20078, "<%s> Failed to create SLM folder [%s]",
    $sMyName, $sSLMDir );
    return 1; # Run SLM..
  }

  #--------------------------
  # Create SLM history file..
  #--------------------------
  if ( sysopen ( SLMFILE, $sSLMFile, O_RDWR|O_SYNC|O_CREAT, 0600 ) ) {
    &logTrace ( 20079,
    "<%s> Opened SLM file [%s] successfully for read/write",
    $sMyName, $sSLMFile );
  }
  else {
    &logWarn ( 20080,
    "<%s> Failed to open SLM file [%s] for read/write",
    $sMyName, $sSLMFile );
    return 1; # Run SLM..
  }

  #---------------
  # Read in file..
  #---------------
  ( $iSLMCntr ) = grep { s/\n// } <SLMFILE>;

  #-----------------
  # Corrupted file..
  #-----------------
  if ( ! defined $iSLMCntr || $iSLMCntr < 0 ) {

    $iSLMCntr ||= ''; # Avoid undefs..

    &logTrace ( 20081,
    "<%s> SLM file [%s] contains [%s]. Re-setting to [%s]",
    $sMyName, $sSLMFile, $iSLMCntr, $iSecondLevelMonitor );
    $iSLMCntr = $iSecondLevelMonitor; # Set to default..
  }

 #-----------------------------------------
  # Update counter, if less than requested..
  #-----------------------------------------
  if ( $iSecondLevelMonitor > $iSLMCntr ) {
    ++ $iSLMCntr;
  }
  else {
    $iSLMCntr = 1;    # Now's the time..
    $iReturnCode = 1; # Run SLM..
  }

  #------------------
  # Update the file..
  #------------------
  seek ( SLMFILE, 0, 0 ); # Rewind before update..

  &logTrace ( 20082,
  "<%s> Updating SLM file [%s] with [%s]. Return code is [%s]",
  $sMyName, $sSLMFile, $iSLMCntr, $iReturnCode );

  print SLMFILE "$iSLMCntr\n"; # Update..

  close ( SLMFILE ) ||
  &logWarn ( 20083,
  "<%s> Close failed on SLM file [%s] after update",
  $sMyName, $sSLMFile, $! );

  return $iReturnCode;
}


=pod
 -------------------------------------------------------------------------
 Function : int SetHostname ( IN String $Hostname ,
                              IN String $ResourceName,
                              IN String $ResourceType,
                              IN String $EntryPointName)

 Description: Sets the hostname of the system to either virtual or physical

 Parameters :

 $sHostname                    Machine Host Name
 $sResourceName                Configured Resource name
 $sResourceType                Type of the configured Resource
 $sEntryPointName              Entry Point Name
 
 Returns : 0 success
           1 failure

 -------------------------------------------------------------------------
=cut

sub SetHostname  {

  #----------------------
  # Get args
  #----------------------
  my ( $sHostname , 
       $sResourceName, 
       $sResourceType, 
       $sEntryPointName) = @_;


  #-----------------
  # Announce Entry..
  #-----------------
  my $sMyName = ( caller 0 )[3];

  &logTrace ( 20084, "Subroutine <%s> called:
  Hostname        [%s],
  ResourceName    [%s],
  ResourceType    [%s],
  EntryPointName  [%s]", 
  $sMyName, $sHostname , $sResourceName, $sResourceType, $sEntryPointName);

  my $stempDir  = "/tmp/.VRTS$sResourceType";
  my $stempFile = "$sResourceName.hostname";

  my $sHostFile="$stempDir/$stempFile";

  if ( $sEntryPointName eq "online" )
  {
    if ( -d $stempDir ) {
      &logTrace ( 20085, "<%s> folder [%s] exists",
      $sMyName, $stempDir );
    }
    else 
    {
      if ( mkdir ( $stempDir, 0700 ) )
      {
        &logTrace ( 20086, "<%s> folder [%s] created.",
        $sMyName, $stempDir );
      }
      else
      {
        &logWarn ( 20087, "<%s> Failed to create temp folder [%s]", 
        $sMyName, $stempDir );
        return 0;
      }
    }
    if ( sysopen ( HOSTFILE, $sHostFile, O_RDWR|O_SYNC|O_CREAT, 0600 ) ) 
    {
      &logTrace ( 20088, "<%s>:Opened hostname file [%s]",
      $sMyName, $sHostFile );
    }
    else
    {
      &logError ( 20089, "<%s>:Unable to open hostname file [%s]",
      $sMyName, $sHostFile );
      return 0;
    }

    #--------------------------------------
    # Save physical host in temporary file
    #--------------------------------------

    my $pHost = `hostname`;
    print HOSTFILE "$pHost\n";

    #--------------------------------------
    # Change hostname to virtual hostname
    #--------------------------------------

   `uname -S $sHostname`;
    return 1;
  }


  if ( $sEntryPointName eq "offline" || $sEntryPointName eq "clean")
  {
    if ( sysopen ( HOSTFILE, $sHostFile, O_RDONLY) ) 
    {
      &logTrace ( 20090, "<%s>:Opened hostname file [%s]",
      $sMyName, $sHostFile );
    }
    else
    {
      &logError ( 20091, "<%s>:Unable to open hostname file [%s]",
      $sMyName, $sHostFile );
      return 0;
    }

    my $pHost = "";
    ( $pHost ) = grep { s/\n// } <HOSTFILE>;

    #------------------------------------
    # Change hostname to virtual hostname
    #------------------------------------

   `uname -S $pHost`;
    return 1;
  }
}

=pod
 -------------------------------------------------------------------------
 Function : int GetEPTimeout ( IN String EntryPointName
                               IN String ResourceType  );

 Description : Returns the Entry Point Timeout for the EP which we are in

 Parameters :

 $sEntryPointName           The EP which we are currently running
 $sResourceType             VCS resource type

 Returns  : Integer value as defined below:

 -------------------------------------------------------------------------
=cut

sub GetEPTimeout
{
  my ( $sEntryPointName, $sResourceType) = @_;

  #------------------
  # Announce Entry..
  #------------------

  my $sMyName = ( caller 0 )[3];
  &logTrace( 20125, "Subroutine <%s> called with:
  EntryPointName [%s]
  ResourceType   [%s]", 
  $sMyName , $sEntryPointName , $sResourceType );

  my $sObjectName          = "hatype";
  my %hTypeValues;

  my $iEngineTimeout;
  #-----------------------------------
  # Query if it has already been set..
  #-----------------------------------
  if ( defined $::giEPTimeout &&    # Defined..
       $::giEPTimeout =~ /^\d+/ ) { # to an integer..
    $iEngineTimeout = $::giEPTimeout;

    &logTrace ( 20126,
    "<%s> Using AEPTimeout [%s] as EP timeout value",
    $sMyName, $iEngineTimeout );
    return ( $iEngineTimeout ); 
  }

  if ( ! VCSagentFW::GetVCSObjectHash($sObjectName,$sResourceType,\%hTypeValues) )
  {
    &logError(20127,"<%s>:GetVCSObjectHash call failed.",$sMyName);
    return 0;
  }

  #-------------------------------
  # Make "monitor" to "Monitor"
  # offline to Offline and so on
  #-------------------------------

  my $sTempEntry =  $sEntryPointName ;
  $sTempEntry    =~ s/(\w)(.*)$/\U$1\L$2/; 

  &logTrace(20128,"<%s>: Returning [%s] value [%s]",$sMyName, $sTempEntry."Timeout",$hTypeValues{$sTempEntry."Timeout"});
  return ( $hTypeValues{$sTempEntry."Timeout"});
}
1; # So PERL knows its included..
